;;###################################################################################################################################
;;#                                                                                                                                 #
;;#     This program is licenced under the Creative Commons (Copyright (c) 2013-2014 IRD / François Rebaudo) except from the        #
;;#     code adapted from David O'Sullivan and George Perry licensed under the Creative Commons (Copyright (c) 2011-13 David        #
;;#     O'Sullivan and George Perry : O'Sullivan D and Perry GLW 2013 _Spatial Simulation: Exploring Pattern and Process_.          #
;;#     Wiley, Chichester, England.).                                                                                               #
;;#                                                                                                                                 #
;;#                                                                                                                                 #
;;###################################################################################################################################

;;---------------------------------------------------------------------------------------------------------------------
;;   EXTENSIONS
;;---------------------------------------------------------------------------------------------------------------------
;; none

;;---------------------------------------------------------------------------------------------------------------------
;;   VARIABLES
;;---------------------------------------------------------------------------------------------------------------------
globals
[  
  output_N_tec               ;; to get the mean infestation as an output file ;Tecia solanivora
  Tecia_solanivora           ;; to allow Tecia solanivora species
  ;output_N_pht              ;; to get the mean infestation as an output file ;Phthorimaea operculella (not used here)
  ;output_N_sym              ;; to get the mean infestation as an output file ;Symmetrischema tangolias (not used here)
  output_Area_tec            ;; to get the area infested as an output file ;Tecia solanivora
  ;output_Area_pht           ;; to get the area infested as an output file ;Phthorimaea operculella (not used here)
  ;output_Area_sym           ;; to get the area infested as an output file ;Symmetrischema tangolias (not used here)
  potato_price               ;; potato price based on time serie analysis 1990-2005 every 6 months (time step for the pest)
  ;sd_potato_price
  coop-change_LOW            ;; intensity of changes when cooperation for each farming style of a farmer
  coop-change_HIGH
  expe-change_LOW            ;; intensity of changes when experimentation for each farming style of a farmer
  expe-change_HIGH
  history-account            ;; number of time steps taken into account for farmer decision-making
  num_farmers                ;; total number of farmers in the landscape
  coop-rate                  ;; rate of cooperation in the community (here, coop-rate = 1)
  maj_group_proba            ;; proportion of farmers strategies (for the dominant strategy)
  
  num-control-drivers        ;; number of control practices (vector of dimension n=5)
  elevation_m                ;; elevation for the landscape
  landscape_resolution       ;; resolution for the landscape in meters (from 50 to 500 to be valid)
  
  stopFromR                  ;; to control the model from R software using RNetLogo package
  
  resilience                 ;; to test resilience of adaptive behavior (at time t, best control practices are modified and we measure time to return to equilibrium)
  
  controlO                   ;; default optimal control practices = list(50,<num-control-drivers times>) with <num-control-drivers> the number of different practices considered
]

patches-own
[
  ;; INSECTS--------------------------------------------------------
  tec                             ;; Tecia solanivora
  ;pht                            ;; Phthorimaea operculella (not used here)
  ;sym                            ;; Symmetrischema tangolias (not used here)
  disp_tec                        ;; to apply dispersal
  ;disp_pht                       ;; to apply dispersal (not used here)
  ;disp_sym                       ;; to apply dispersal (not used here)
  
  ;; LANDSCAPE------------------------------------------------------
  temperature                     ;; temperature in Celsius degrees 
  elevation                       ;; elevation in m.a.s.l.
  landuse                         ;; localisation of crops (0=nothing, 1=crop)
  carrying_capacity               ;; carrying capacity for insects
  storage_structure               ;; localization of storage structures
  
  ;; SOCIAL-SYSTEM--------------------------------------------------
  farm_owner                      ;; to initiate parcels
]

breed [farmers farmer]

farmers-own [
  voronoi-polygon      ;; fields attributed to a farmer
  histoINF             ;; infestation over the last <num_generations> years: (list <ticks> mean(<tec>) sd(<tec>))
  coop                 ;; if farmer cooperate = 1 else = 0 (allways 1 in this paper)
  farming-style        ;; farming styles of a farmer ("conv" "NOrisk" "risk" "expe" "NOchange")
  control-ope          ;; 100 - sqrt(SUM(diff^2 between controlP and controlO)) of a farmer => 100: optimal control, and 0: worst control
  controlP             ;; current control practices of a farmer: (list x ... x)
  histoCONTROL         ;; control practices <controlP> over the last <num_generations> years: (list (list mean(<control-ope>) ... mean(<control-ope>)) (list <controlP> ... <controlP>))
  areaFarm             ;; area of the farm in ha
]

;;---------------------------------------------------------------------------------------------------------------------
;;   INITIALIZATION
;;---------------------------------------------------------------------------------------------------------------------
to setup_init  ;; performed once for a set of simulations
  __clear-all-and-reset-ticks                                ;; clear everything from previous runs
  set repet 1                                                ;; set the number of repetition to 1
  set Tecia_solanivora True                                  ;; to allow the pest
  set num-control-drivers 5                                  ;; 5 control practices
  set controlO n-values num-control-drivers [50]             ;; set the theoretical optimum control practices to 50 [0:100]
  set stopFromR False                                        ;; a boolean which turn TRUE when simulations are over so that the model can stop the WHILE loop from R using RNetLogo package
  resize-world 0 (world-width-def - 1) 0 (world-height - 1)  ;; set world-width of the landscape (sensibility analysis on the number of farmers)
  set num_farmers world-width * world-height * 0.15          ;; to keep an average farm size of 1.6 ha
  set coop-rate 1                                            ;; every farmers cooperate
  set landscape_resolution 50                                ;; landscape resolution to fit farm size
  set maj_group_proba 0.5                                    ;; the dominant strategy is adopted by 50% of farmers, and equally for remaining strategies
  set coop-change_LOW 2                                      ;; intensity of changes when mimicking
  set coop-change_HIGH 4                                     ;; intensity of changes when mimicking
  set expe-change_LOW 8                                      ;; intensity of changes when experimentating
  set expe-change_HIGH 10                                    ;; intensity of changes when experimentating
  set history-account 4                                      ;; 4 time steps = 2 years for learning assemssment
  
  ;;set social-net-range 100                                 ;; not used
  set resilience 5000                                        ;; if ticks = resilience, best control practices change to 0 instead of 50
  setup                                                      ;; setup for each simulation
end

to setup  ;; performed once for each simulation
  clear-turtles  ;; clear farmer agents

  set_temp_preci                       ;; setting temperature and precipitation at initialization             
  set_elevation                        ;; loading elevation for all patches                                  
  set_landuse                          ;; loading the localisation of crops
  set_temp_adjust                      ;; adjust temperature in storage structures according to elevation and outside temperature   
  set_K                                ;; setting carrying capacity for all patches                          
  set_moths                            ;; setting insects in patches     
  
  set_farmers                                          ;; create the landscape with fields, create the farmer agents and attribute fields to farmer agents
  set potato_price random-normal 100 sd_potato_price   ;; set potato price for the first generation
  ask farmers[set areaFarm (count(patches with [farm_owner = myself]) * landscape_resolution * landscape_resolution) / 10000]
end

;;---------------------------------------------------------------------------------------------------------------------
;;   MAIN PROGRAM
;;---------------------------------------------------------------------------------------------------------------------
to go
  tick

  if ticks = resilience[
    set controlO n-values num-control-drivers [25]
  ]
  
  ;; (A) LANDSCAPE: set temperature for the current time step
  ;;###################################################################################################################
  set_temp_preci  ;; loading temperatures (not only in setup so that a stochastic factor can be added, or a global warming scenario...)
  set_temp_adjust ;; adjusting temperatures in storage structures (no storage structure in the landscape)
  
  ;; (B) PEST: reproduction
  ;;###################################################################################################################
  main_insect_reproduction

  ;; (C) PEST: survival
  ;;###################################################################################################################
  main_insect_survival
  
  ;; (D) FARMERS: adaptive pest control
  ;;###################################################################################################################
  farmer_adaptive_control
  ;;show count links

  ;; (E) PEST: dispersion
  ;;###################################################################################################################
  main_insect_dispersion
  
  ;; (F) PEST: human induced dispersion
  ;;###################################################################################################################
  ;; no

  ;; (G) POTATO PRICE
  ;;###################################################################################################################
  set potato_price random-normal 100 sd_potato_price

  ;; (H) MODEL: generations and repetitions
  ;;###################################################################################################################
  if ticks = num_generations [
    
    ;; FILE WITH TEC INFESTATION
    ;;#######################
    let fichier ""
    ifelse repet < 10 [set fichier (word working_directory "tec_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_00" repet "_temp" temperature_m "_" behaviorspace-run-number ".csv")][
      if repet < 100 [set fichier  (word working_directory "tec_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_0"  repet "_temp" temperature_m "_" behaviorspace-run-number ".csv")]
    ]
    let tecfile ""
    ask farmers [
      set tecfile (word tecfile self ";" farming-style ";")
      let rev reverse item 1 histoINF
      foreach rev [set tecfile (word tecfile ? ";")]
      set tecfile (word tecfile "\n")
    ]
    file-open fichier
      file-print tecfile
    file-close
    
    ;; FILE WITH FARMERS MEAN CONTROL
    ;;#######################
    set fichier ""
    ifelse repet < 10 [set fichier (word working_directory "practices_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_00" repet "_temp" temperature_m "_size_" world-width-def "_" behaviorspace-run-number ".csv")][
      if repet < 100 [set fichier  (word working_directory "practices_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_0"  repet "_temp" temperature_m "_size_" world-width-def "_" behaviorspace-run-number ".csv")]
    ]
    let farmfile ""
    ask farmers [
        set farmfile (word farmfile self ";" farming-style ";")
        let rev reverse item 0 histoCONTROL
        foreach rev [set farmfile (word farmfile ? ";")]
        set farmfile (word farmfile "\n")
      ]
    file-open fichier
      file-print farmfile
    file-close
    
    ;; FILE WITH FARMERS PRACTICES
    ;;#######################
    set fichier ""
    ifelse repet < 10 [set fichier (word working_directory "practicesReal_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_00" repet "_temp" temperature_m "_size_" world-width-def "_" behaviorspace-run-number ".csv")][
      if repet < 100 [set fichier  (word working_directory "practicesReal_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_0"  repet "_temp" temperature_m "_size_" world-width-def "_" behaviorspace-run-number ".csv")]
    ]
    let pracfile ""
    ask farmers [
        set pracfile (word pracfile self ";" farming-style ";")
        let rev reverse item 1 histoCONTROL
        foreach rev [set pracfile (word pracfile ? ";")]
        set pracfile (word pracfile "\n")
      ]
    file-open fichier
      file-print pracfile
    file-close
    
    ;; FILE WITH FARMERS NETWORK
    ;;#######################
    set fichier ""
    ifelse repet < 10 [set fichier (word working_directory "network_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_00" repet "_temp" temperature_m "_size_" world-width-def "_" behaviorspace-run-number ".csv")][
      if repet < 100 [set fichier  (word working_directory "network_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_0"  repet "_temp" temperature_m "_size_" world-width-def "_" behaviorspace-run-number ".csv")]
    ]
    export-plot "network" fichier
    
    ;; FILE WITH SPATIAL COO.
    ;;#######################
    set fichier (word working_directory "spatial_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_0" repet "_temp" temperature_m "_size_" world-width-def "_" behaviorspace-run-number ".csv")
    let spafile ""
    ask farmers [
      set spafile (word spafile xcor ";" ycor ";" )
      foreach controlP [set spafile (word spafile ? ";")]
      set spafile (word spafile "\n")
    ]
    file-open fichier
      file-print spafile
    file-close
    
  ]
  
  if ticks = num_generations [  ;; at the end of "num_generations", a new simulation starts and "repet" is incremented
    set repet repet + 1
    reset-ticks
    setup  
  ]
  if (repet - 1) = num_repetitions [  ;;at the end of "repet", the set of simulations is over and procedure "go" stops
    set stopFromR True
    stop
  ]
  
  let fichier ""
  ifelse repet < 10 [set fichier (word working_directory "histo_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_00" repet "_" ticks "_size_" world-width-def "_" behaviorspace-run-number ".txt")][
    if repet < 100 [set fichier  (word working_directory "histo_" maj_group "_sdprix_" precision sd_potato_price 3 "_sdtemp_" precision sd_temperature_m 3 "_0" repet "_" ticks "_size_" world-width-def "_" behaviorspace-run-number ".txt")]
  ]
  
  if (ticks mod 50 = 0) [
  ;;if ticks = num_generations or ticks = (num_generations / 2)[
    let stringfile (word "cop" ticks "<-list(c(")
    ask farmers with[farming-style = "conv"] [set stringfile (word stringfile mean(control-ope) ",")]
    set stringfile (word stringfile "),c(")
    ask farmers with[farming-style = "NOrisk"] [set stringfile (word stringfile mean(control-ope) ",")]
    set stringfile (word stringfile "),c(")
    ask farmers with[farming-style = "risk"] [set stringfile (word stringfile mean(control-ope) ",")]
    set stringfile (word stringfile "),c(")
    ask farmers with[farming-style = "expe"] [set stringfile (word stringfile mean(control-ope) ",")]
    set stringfile (word stringfile "),c(")
    ask farmers with[farming-style = "NOchange"] [set stringfile (word stringfile mean(control-ope) ",")]
    set stringfile (word stringfile "))\n")
    
    file-open fichier
      file-print stringfile
    file-close
  ]
end

;;---------------------------------------------------------------------------------------------------------------------
;;   SUBMODELS
;;---------------------------------------------------------------------------------------------------------------------
     ;;----------------------------------------------------------------------------------------------------------------
     ;;   INITIALIZATION SUB-MODELS
     ;;----------------------------------------------------------------------------------------------------------------
     
   to-report report_farming-style_distrib [my_maj_group] 
     let farming-styles (list "conv" "NOrisk" "risk" "expe" "NOchange")
     let farming-styles-m filter [? != my_maj_group] farming-styles
     ifelse random-float 1.0 < maj_group_proba [
       set farming-style my_maj_group
     ][
       let i random 4
       ifelse i = 0 [
           set farming-style item 0 farming-styles-m 
         ][
           ifelse i = 1 [
             set farming-style item 1 farming-styles-m 
           ][
             ifelse i = 2 [
               set farming-style item 2 farming-styles-m 
             ][
               ifelse i = 3 [
                 set farming-style item 3 farming-styles-m 
               ][
                 print "error in farming-style definition"
               ]
             ]  
           ]
         ]
     ]
     report 0
   end
   
   to set_farmers_own  ;; set the farmers state-variables: farming-style, history-account, coop-change, expe-change
         ifelse random-float 1 < coop-rate [set coop 1][set coop 0]
         set farming-style "none"     ;; initialize without farming-style
         
         let j report_farming-style_distrib maj_group ;; determine the dominant group in the community, fixed to 50% (see Paredes 2010)
         
         let H num_generations
         set histoINF (list n-values H [0] n-values H [0] n-values H [0])       ;; to store "0=ticks 1=infestation(mean) 2=infestation(sd)" history in a list of lists for the last H generations
         ;set controlP n-values num-control-drivers [random 100]                ;; t0 = random default control level at initialization => now according to observed distribution, see Rebaudo et al. 2011
         let jj 0
         let distrib_k (list 0.058 0.444 0.874 0.976 1 1)                       ;; according to observed distribution, see Rebaudo et al. 2011 figure 1
         set controlP (list)
         while [jj < num-control-drivers] [                                     ;; attribution of control practices with 50=optimal and 0 and 100 worst values. Attribution according to "distrib_k" with sd=1
           let xx 100
           let tirage random-float 1.0
           if tirage < item 5 distrib_k [ifelse random 2 = 1 [set xx precision random-normal  50 1 0][set xx precision random-normal  50 1 0]]
           if tirage < item 4 distrib_k [ifelse random 2 = 1 [set xx precision random-normal  60 1 0][set xx precision random-normal  40 1 0]]
           if tirage < item 3 distrib_k [ifelse random 2 = 1 [set xx precision random-normal  70 1 0][set xx precision random-normal  30 1 0]]
           if tirage < item 2 distrib_k [ifelse random 2 = 1 [set xx precision random-normal  80 1 0][set xx precision random-normal  20 1 0]]
           if tirage < item 1 distrib_k [ifelse random 2 = 1 [set xx precision random-normal  90 1 0][set xx precision random-normal  10 1 0]]
           if tirage < item 0 distrib_k [ifelse random 2 = 1 [set xx precision random-normal 100 1 0][set xx precision random-normal   0 1 0]]
           if xx < 0 [set xx 0]
           if xx > 100 [set xx 100]
           
           set controlP fput xx controlP
           set jj jj + 1
         ]
         set histoCONTROL (list n-values H [0] n-values H [0])                  ;; to store deltaControl and control strategy history in a list of lists for the last H generations
   end
   
   to set_farmers ;; create the landscape with fields, create the farmer agents and attribute fields to farmer agents
     ;; this part of the code is adapated from David O'Sullivan and George Perry licensed under the Creative Commons (Copyright (c) 2011-13 David O'Sullivan and George Perry)------------INIT
       ask patches [ set farm_owner nobody ]
       let palette n-values 139 [random 139]
       create-farmers num_farmers [
         set size 0.5
         set shape "circle"
         set color one-of palette
         setxy random-xcor random-ycor
         set voronoi-polygon patch-set nobody
         set_farmers_own ;; set the farmers state-variables: farming-style, history-account, coop-change, expe-change
       ]
       ask patches with [landuse = 1][ 
         let N-farm_owner filter [? != nobody] ([farm_owner] of neighbors4)
         ifelse length N-farm_owner = 4 and length remove-duplicates N-farm_owner = 1 [
           set farm_owner one-of N-farm_owner
         ][ 
           set farm_owner min-one-of farmers [distance myself] 
         ]
         set pcolor [color] of farm_owner
         ;; add this patch to the voronoi polygon of the parent
         ask farm_owner [
           set voronoi-polygon (patch-set voronoi-polygon myself)
         ]
       ]
       ask farmers[
         let n-patches patch-set [neighbors4] of voronoi-polygon
         let n-polys turtle-set [farm_owner] of n-patches
         ask other n-polys [
           create-link-with myself []
         ]
       ]
    ;; end of the code adapted from David O'Sullivan and George Perry---------------------------------------------------------------------------------------------------------------------END
  
    let countlist (list)
    ask farmers[
      set color 0
      let a landscape_resolution ^ 2 * count(patch-set voronoi-polygon)
      if a = 0 [die]
      set countlist lput a countlist
    ]
    type precision (mean countlist / 10000) 2 type " ha per farmer (sd: " type precision (standard-deviation countlist / 10000) 2 type "). max: " type max countlist / 10000 type " ha and min: " type min countlist / 10000 type " ha. (" type length countlist print " farmers)"
   end
  
     
     to set_elevation
       ask patches [set elevation elevation_m]
     end
     
     to set_landuse
       ask patches [
         ifelse random-float 1 < 0.05 [
           set landuse 0
         ] [
           set landuse 1
         ]
         
       ]
     end

     
     to set_temp_adjust ;; temperature in storage structures depending on elevation and outside temperature, cf. Crespo-Perez et al. 2011 Landscape Ecology
       ask patches with [storage_structure = 1] [
         let a 0
         let b 0
         ifelse elevation < 2899 [
           set a -0.144
           set b 13.552
         ] [
           ifelse elevation < 2999 [
             set a -0.144
             set b 13.552
           ] [
             ifelse elevation < 3099 [
               set a -0.192
               set b 13.598
             ] [
               ifelse elevation < 3199 [
                 set a -0.183
                 set b 12.636
               ] [
                 ifelse elevation < 3299 [
                   set a -0.079
                   set b 9.864
                 ] [
                   set a -0.021
                   set b 8.893
                 ]
               ]
             ]
           ]
         ]
         set temperature (a * temperature + b)
       ]
     end
       
     to set_K
       ask patches [
         set carrying_capacity (landscape_resolution ^ 2) * 1000 / (500 ^ 2)  ;; adapted from Crespo-Perez et al. 2011 Landscape Ecology
       ]
     end
     
     to set_temp_preci
       let tx_temp random-normal temperature_m sd_temperature_m  ;; homogeneous temperature taken in a gaussian distribution with sd=sd_temperature_m (in the GUI; from 0)
       ask patches [
         set temperature tx_temp  
         if temperature < 0 [set temperature 0]
       ]
     end
     
     to set_moths
       ask patches [
         set tec 0
         ;set pht 0                                        ;; ###PHT
         ;set sym 0                                        ;; ###SYM
       ]
       ask patches ;0 round(world-height  / 2)
       [
         ifelse Tecia_solanivora = true [
           set tec 10                                           ;; no significant effect on infestation levels (see Crespo-Perez et al. 2011 Landscape Ecology)
         ] [
           set tec 0
         ]
         ;ifelse Phthorimaea_operculella = true [                                         ;; ###PHT
         ;  set pht 10
         ;] [
         ;  set pht 0
         ;]
         ;ifelse Symmetrischema_tangolias = true [                                        ;; ###SYM
         ;  set sym 10
         ;] [
         ;  set sym 0
         ;]
       ]
     end


     ;;----------------------------------------------------------------------------------------------------------------
     ;;   FARMERS SUB-MODELS
     ;;----------------------------------------------------------------------------------------------------------------
to updateFarmerHistory
  ask farmers [
    ;; updating history
    let meanINF 0
    let sdINF 0
    ifelse count(patches with [farm_owner = myself]) > 1 [set meanINF precision mean([tec] of patches with [farm_owner = myself]) 2][set meanINF precision max([tec] of patches with [farm_owner = myself]) 0]        ;; mean level of tec per farm
    ifelse count(patches with [farm_owner = myself]) > 1 [set sdINF precision standard-deviation([tec] of patches with [farm_owner = myself]) 2][set sdINF 0]                                                         ;; sd level of tec per farm

    ;; [1] updating history of infestation levels
    set histoINF replace-item 0 histoINF (fput ticks   item 0 histoINF)
    set histoINF replace-item 0 histoINF (but-last item 0 histoINF)
    set histoINF replace-item 1 histoINF (fput meanINF item 1 histoINF)
    set histoINF replace-item 1 histoINF (but-last item 1 histoINF)
    set histoINF replace-item 2 histoINF (fput sdINF   item 2 histoINF)
    set histoINF replace-item 2 histoINF (but-last item 2 histoINF)
    
    ;; [2] updating history of control practices
    set histoCONTROL replace-item 0 histoCONTROL (fput mean control-ope item 0 histoCONTROL)
    set histoCONTROL replace-item 0 histoCONTROL (but-last              item 0 histoCONTROL)
    set histoCONTROL replace-item 1 histoCONTROL (fput controlP item 1 histoCONTROL)
    set histoCONTROL replace-item 1 histoCONTROL (but-last      item 1 histoCONTROL)
  ]
end

to farmer_adaptive_control
  ;; FARMERS: control action (FIXED BEHAVIOR= farmers always have to control the pest every crop cycle)
  ask farmers [
    let i 0
    set control-ope (list)
    while [i < num-control-drivers] [
      let j sqrt((item i controlP - item i controlO) ^ 2)      ;; control-ope = (50 - absolute difference between optimal control and farmer control)*2 so that control-ope = [0:100]
      set control-ope lput ((50 - j) * 2) control-ope
      set i i + 1
    ]
    
    let mycontrol mean control-ope / 20                        ;; take the average value of control-ope and change the scale from 0 to 100 into a scale from 0 to 5, see Rebaudo et al. 2011 (doi:10.1371/journal.pcbi.1002222)
    if (mycontrol > 5) [set mycontrol 5]
    if (mycontrol < 0) [set mycontrol 0]
    ask patches with [farm_owner = myself] [
      ;let epsilon (random-float 0.08)                          ;; variability estimation ~8% based on confidence intervals when k=2 (raw data of figure 1 from Rebaudo et al. 2011)
      let epsilon 0                                            ;; without variability
      set tec tec - tec * (0.15 * mycontrol + 0.21 + epsilon)  ;; with mycontrol = [0:5], see Rebaudo et al. 2011 (doi:10.1371/journal.pcbi.1002222)
      if tec < 0 [set tec 0]                                   ;; to avoid negative values
    ]
  ]
  
  updateFarmerHistory
  
  ask farmers [
    
  ;; FARMERS: modification of control action for next crop (ADAPTIVE-REFLECTIVE BEHAVIOR= control practices change dynamically and according to history)
    
    
    ;; BASED ON PEST INFESTATION AS AN INDIRECT EVALUATION OF YIELD ;; mean infestation history = item 1 histoINF ; (K - infestation)/K = estimation of yield
    
    ;; [1 OPTION 1] ;; choose another farmer with "my mean yield over X years" > "his/her mean yield over X years"
    ;;let coop-farmer-history_m1i1 one-of (turtles with [                                                      ;; without social range
    ;;let coop-farmer-history_m1i1 one-of ((turtles-on patches in-radius (social-net-range)) with [            ;; with distance social range
    let coop-farmer-history_m1i1 one-of (link-neighbors with [                                                 ;; with node social range
      (self != myself) and 
      (carrying_capacity - mean(sublist(item 1 histoINF) 0 history-account) / carrying_capacity) > (carrying_capacity - [mean(sublist(item 1 histoINF) 0 history-account)] of myself / carrying_capacity) 
    ])

    ;; BASED ON ECONOMIC RETURN ;; yield * potato_price
    
    ;; [1 OPTION 3] ;; choose another farmer with "my actual economic return over X years" > "his/her actual economic return over X years"
    ;;let coop-farmer-history_m1i1_dol one-of (turtles with [                                                      ;; without social range
    ;;let coop-farmer-history_m1i1_dol one-of ((turtles-on patches in-radius (social-net-range)) with [            ;; with distance social range
    let coop-farmer-history_m1i1_dol one-of (link-neighbors with [                                                 ;; with node social range
      (self != myself) and 
      (carrying_capacity - mean(sublist(item 1 histoINF) 0 history-account) / carrying_capacity) * potato_price > (carrying_capacity - [mean(sublist(item 1 histoINF) 0 history-account)] of myself / carrying_capacity) * potato_price 
    ])
    
    

    ;; [2] ;; adaptive management 
    if farming-style = "conv" [ ;; adaptive management according to OTHERS (if any), based on ECONOMIC RETURN and accounting for HISTORY (h=X)
      if is-turtle? coop-farmer-history_m1i1_dol [
        ask link-with coop-farmer-history_m1i1_dol [set color 15]
        let coop-farmer-control 0
        ask coop-farmer-history_m1i1_dol [set coop-farmer-control controlP]
        let ii 0
        while [ii < coop-change_HIGH] [
          let item2change random num-control-drivers
          ifelse item item2change coop-farmer-control - item item2change controlP > 0 [
            set controlP replace-item item2change controlP (item item2change controlP + 1)
            
            if (item item2change controlP + 1 > 100)[set controlP replace-item item2change controlP (100)]
            
          ][
            ;; if item = item then nothing
            if item item2change coop-farmer-control - item item2change controlP < 0 [
              set controlP replace-item item2change controlP (item item2change controlP - 1)
              
              if (item item2change controlP - 1 < 0)[set controlP replace-item item2change controlP (0)]
              
            ]
          ]
          set ii ii + 1
        ]
      ]
    ]
    if farming-style = "NOrisk" [ ;; adaptive management according to OTHERS (if any), based on YIELD and accounting for HISTORY (h=X)
      if is-turtle? coop-farmer-history_m1i1 [
        ask link-with coop-farmer-history_m1i1 [set color 15]
        let coop-farmer-control 0
        ask coop-farmer-history_m1i1 [set coop-farmer-control controlP]
        let ii 0
        while [ii < coop-change_LOW] [
          let item2change random num-control-drivers
          ifelse item item2change coop-farmer-control - item item2change controlP > 0 [
            set controlP replace-item item2change controlP (item item2change controlP + 1)
            
            if (item item2change controlP + 1 > 100)[set controlP replace-item item2change controlP (100)]
            
          ][
            ;; if item = item then nothing
            if item item2change coop-farmer-control - item item2change controlP < 0 [
              set controlP replace-item item2change controlP (item item2change controlP - 1)
              
              if (item item2change controlP - 1 < 0)[set controlP replace-item item2change controlP (0)]
              
            ]
          ]
          set ii ii + 1
        ]
      ]
    ]
    if farming-style = "risk" [ ;; adaptive management according to EXTERNAL RECOMMENDATIONS, based on ECONOMIC RETURN, accounting for OWN HISTORY (h=1)
      
      ifelse ((carrying_capacity - item 0 (item 1 histoINF)) / carrying_capacity) * potato_price < ((carrying_capacity - item 1 (item 1 histoINF)) / carrying_capacity) * potato_price [
      ;; if before better than now, then WRONG recommendation has been made: change from PREVIOUS (n-1) practices
        if ticks > 1 [set controlP item 1 (item 1 histoCONTROL)]
        let dirChange random-float 1.0
        let ii 0
        while [ii < expe-change_HIGH] [
          let item2change random num-control-drivers
          ifelse dirChange < 0.5 [
            set controlP replace-item item2change controlP (item item2change controlP + 1)
            
            if (item item2change controlP + 1 > 100)[set controlP replace-item item2change controlP (100)]
            
          ][
            set controlP replace-item item2change controlP (item item2change controlP - 1)
            
            if (item item2change controlP - 1 < 0)[set controlP replace-item item2change controlP (0)]
            
          ]
          set ii ii + 1
        ]
      ][
      ;; if before worst than now, then GOOD recommendation has been made: change from ACTUAL practices practices
        let dirChange random-float 1.0
        let ii 0
        while [ii < expe-change_HIGH] [
          let item2change random num-control-drivers
          ifelse dirChange < 0.5 [
            set controlP replace-item item2change controlP (item item2change controlP + 1)
            
            if (item item2change controlP + 1 > 100)[set controlP replace-item item2change controlP (100)]
            
          ][
            set controlP replace-item item2change controlP (item item2change controlP - 1)
            
            if (item item2change controlP - 1 < 0)[set controlP replace-item item2change controlP (0)]
            
          ]
          set ii ii + 1
        ]
      ]
    ]
    if farming-style = "expe" [ ;; adaptive management according to own EXPERIMENTATION, based on YIELD, accounting for OWN HISTORY (h=1)
      
      ifelse ((carrying_capacity - item 0 (item 1 histoINF)) / carrying_capacity) < ((carrying_capacity - item 1 (item 1 histoINF)) / carrying_capacity) [
      ;; if before better than now, then WRONG experimentation has been made: change from PREVIOUS (n-1) practices
        if ticks > 1 [set controlP item 1 (item 1 histoCONTROL)]
        let dirChange random-float 1.0
        let ii 0
        while [ii < expe-change_LOW] [
          let item2change random num-control-drivers
          ifelse dirChange < 0.5 [
            set controlP replace-item item2change controlP (item item2change controlP + 1)
            
            if (item item2change controlP + 1 > 100)[set controlP replace-item item2change controlP (100)]
            
          ][
            set controlP replace-item item2change controlP (item item2change controlP - 1)
            
            if (item item2change controlP - 1 < 0)[set controlP replace-item item2change controlP (0)]
            
          ]
          set ii ii + 1
        ]
      ][
      ;; if before worst than now, then GOOD experimentation has been made: change from ACTUAL practices practices
        let dirChange random-float 1.0
        let ii 0
        while [ii < expe-change_LOW] [
          let item2change random num-control-drivers
          ifelse dirChange < 0.5 [
            set controlP replace-item item2change controlP (item item2change controlP + 1)
            
            if (item item2change controlP + 1 > 100)[set controlP replace-item item2change controlP (100)]
            
          ][
            set controlP replace-item item2change controlP (item item2change controlP - 1)
            
            if (item item2change controlP - 1 < 0)[set controlP replace-item item2change controlP (0)]
            
          ]
          set ii ii + 1
        ]
      ]
    ]
    if farming-style = "NOchange" [] ;; no adaptive management

  ]
end
     
     
     ;----------------------------------------------------------------------------------------------------------------
     ;   INSECT SUB-MODELS
     ;----------------------------------------------------------------------------------------------------------------
     
     
          ;-----------------------------------------------------------------------------------------------------------
          ;   INSECT SUB-MODELS : REPRODUCTION
          ;-----------------------------------------------------------------------------------------------------------
     to main_insect_reproduction
       ;(A) mating rate
       ;ok 100%
       
       ;(B) sex ratio
       ;ok 1:1
       
       ;(C) fecundity 
       ;ask patches with [tec + sym + pht > 0] [    ;; ### PHT and SYM
       ask patches with [tec > 0] [
         let fecundity 0
         ;set fecundity report_fecundity -21.62 345.5 18.66 0.32 243 ;cf. Crespo-Perez et al. 2011 Landscape Ecology
         set fecundity report_fecundity -30.118 343.281 18.753 17.109 3.300 ;cf. Rebaudo et al. 2011 JASSS
         set tec tec * fecundity
         ;set fecundity report_fecundity -3468.926 3606.259 21.072 220.998 4.706    ;; ### PHT and SYM
         ;set pht pht * fecundity                                                   ;; ### PHT and SYM
         ;set fecundity report_fecundity -2.062 218.456 15.108 19.484 2.572         ;; ### PHT and SYM
         ;set sym sym * fecundity                                                   ;; ### PHT and SYM
       ]
     end
     
     to-report report_fecundity [o p q r s] ;using the gamma function as in Crespo-Perez et al. 2011 Landscape Ecology
       let fecundity o + (p * exp (- (temperature - q) / r) * ((((temperature - q) / r) + (s - 1)) / (s - 1))^(s - 1))
       if fecundity < 0 [set fecundity 0]
       report fecundity
     end
     
     to-report report_fecundity_alt [aa bb cc dd ee] ;alternative to the gamma function which was not suitable for pht and sym = Weibull distribution function
       let fecundity aa + bb * exp ( - ( ( ( temperature - cc ) / dd ) + ( ( ee - 1 ) / ee ) ^ ( 1 / ee ) ) ^ ee + ( ( ee - 1 ) / ee ) ) * ( ( ee - 1 ) / ee ) ^ ( - ( ( ee - 1 ) / ee ) ) * ( ( ( temperature - cc ) / dd ) + ( ( ee - 1 ) / ee ) ^ ( 1 / ee ) ) ^ ( ee - 1 )
       if fecundity < 0 [set fecundity 0]
       report fecundity
     end
          
          ;;-----------------------------------------------------------------------------------------------------------
          ;;   INSECT SUB-MODELS : SURVIVAL
          ;;-----------------------------------------------------------------------------------------------------------
     to main_insect_survival
       ;ask patches with [tec + sym + pht > 0 and landuse = 0] [    ;; ### PHT and SYM
       ask patches with [tec > 0 and landuse = 0] [
         set tec 0
         ;set pht 0                                                 ;; ### PHT and SYM
         ;set sym 0                                                 ;; ### PHT and SYM
       ]
       ;ask patches with [tec + sym + pht > 0] [                    ;; ### PHT and SYM
       ask patches with [tec > 0] [
         
         ;; (A) temperature dependent mortality using Sharpe and De Michel equation
         let survival_tec set_survival_tec     ;; cf. Rebaudo et al. 2011 JASSS
         ;let survival_pht set_survival_pht    ;; cf. Rebaudo et al. 2011 JASSS    ;; ### PHT and SYM
         ;let survival_sym set_survival_sym    ;; cf. Rebaudo et al. 2011 JASSS    ;; ### PHT and SYM
         
         ;; (B) demographically based mortality (occuring between each immature stage, i.e. from egg to larva, from larva to pupa and from pupa to adults)
         set survival_tec survival_tec * (demo_survival ^ 3)     ;; cf. Crespo-Perez et al. 2011 Landscape Ecology
         ;set survival_pht survival_pht * (demo_survival ^ 3)    ;; Assuming similarity to Tecia, cf. Crespo-Perez et al. 2011 Landscape Ecology    ;; ### PHT and SYM
         ;set survival_sym survival_sym * (demo_survival ^ 3)    ;; Assuming similarity to Tecia, cf. Crespo-Perez et al. 2011 Landscape Ecology    ;; ### PHT and SYM
         
         ;; (C) adjustment of moth generation length = f(temperature) using Sharpe and De Michel equation
         let adjust_tec set_adjust_tec     ;; cf. Crespo-Perez et al. 2011 Landscape Ecology
         ;let adjust_pht set_adjust_pht    ;; ### PHT and SYM
         ;let adjust_sym set_adjust_sym    ;; ### PHT and SYM
         
         let adjust_relatif_tec 1
         ;let adjust_relatif_pht adjust_pht / adjust_tec    ;; ### PHT and SYM
         ;let adjust_relatif_sym adjust_sym / adjust_tec    ;; ### PHT and SYM

         ;; observation: for hot temperatures, the adjustement values reach extreme values (100 and more) to avoid such situation an IF control should be added on the program depending on the landscape
         ;if adjust_relatif_pht > 1.5 [set adjust_relatif_pht 1.5]
         ;if adjust_relatif_sym > 1.5 [set adjust_relatif_sym 1.5]
         
         ;; (D) Updating number of insects with survival
         set tec tec * survival_tec
         ;set pht pht * survival_pht    ;; ### PHT and SYM
         ;set sym sym * survival_sym    ;; ### PHT and SYM

         ;; (E) Updating number of insects with adjustments
         set tec tec * adjust_relatif_tec
         ;set pht pht * adjust_relatif_pht    ;; ### PHT and SYM
         ;set sym sym * adjust_relatif_sym    ;; ### PHT and SYM

         ;; (F) precipitation dependent mortality
         let precipitation_survival 1
         
         ;; (G) Updating number of insects with rainfall
         set tec tec * precipitation_survival
         ;set pht pht * precipitation_survival    ;; ### PHT and SYM
         ;set sym sym * precipitation_survival    ;; ### PHT and SYM
       ]
       ;limiting population to the carrying capacity
       ;ask patches with [tec > carrying_capacity or sym > carrying_capacity or pht > carrying_capacity] [     ;; ### PHT and SYM
       ask patches with [tec > carrying_capacity] [
         if tec > carrying_capacity [set tec carrying_capacity]
         ;if pht > carrying_capacity [set pht carrying_capacity]     ;; ### PHT and SYM
         ;if sym > carrying_capacity [set sym carrying_capacity]     ;; ### PHT and SYM
       ]
     end
     
     to-report set_adjust_sym
       let temp_kelvin temperature + 273.15
       let r_constant 1.987
       let a_var_egg 0.185       let b_var_egg 17250       let c_var_egg -48000       let d_var_egg 280.0       let e_var_egg 121830       let f_var_egg 303.5
       let a_var_larva 0.050     let b_var_larva 11000     let c_var_larva -50000     let d_var_larva 282.1     let e_var_larva 275000     let f_var_larva 305.1
       let a_var_pupa 0.094      let b_var_pupa 11500      let c_var_pupa -35000      let d_var_pupa 283.2      let e_var_pupa 125000      let f_var_pupa 302.2
       report report_survival r_constant temp_kelvin a_var_egg b_var_egg c_var_egg d_var_egg e_var_egg f_var_egg a_var_larva b_var_larva c_var_larva d_var_larva e_var_larva f_var_larva a_var_pupa b_var_pupa c_var_pupa d_var_pupa e_var_pupa f_var_pupa
     end
     
     to-report set_adjust_pht
       let temp_kelvin temperature + 273.15
       let r_constant 1.987
       let a_var_egg 0.190       let b_var_egg 17250       let c_var_egg -48000       let d_var_egg 278.1       let e_var_egg 121830       let f_var_egg 303;
       let a_var_larva 0.070     let b_var_larva 11000     let c_var_larva -50000     let d_var_larva 287.5     let e_var_larva 275000     let f_var_larva 305;
       let a_var_pupa 0.119      let b_var_pupa 11500      let c_var_pupa -35000      let d_var_pupa 285.5      let e_var_pupa 125000      let f_var_pupa 302;
       report report_survival r_constant temp_kelvin a_var_egg b_var_egg c_var_egg d_var_egg e_var_egg f_var_egg a_var_larva b_var_larva c_var_larva d_var_larva e_var_larva f_var_larva a_var_pupa b_var_pupa c_var_pupa d_var_pupa e_var_pupa f_var_pupa
     end
     
     to-report set_adjust_tec
       let temp_kelvin temperature + 273.15
       let r_constant 1.987
       let a_var_egg 0.179       let b_var_egg 17250       let c_var_egg -48000       let d_var_egg 265.2       let e_var_egg 121830       let f_var_egg 304.2
       let a_var_larva 0.076     let b_var_larva 11000     let c_var_larva -50000     let d_var_larva 283.1     let e_var_larva 275000     let f_var_larva 302.1
       let a_var_pupa 0.187      let b_var_pupa 11500      let c_var_pupa -35000      let d_var_pupa 290.0      let e_var_pupa 125000      let f_var_pupa 299.5
       report report_survival r_constant temp_kelvin a_var_egg b_var_egg c_var_egg d_var_egg e_var_egg f_var_egg a_var_larva b_var_larva c_var_larva d_var_larva e_var_larva f_var_larva a_var_pupa b_var_pupa c_var_pupa d_var_pupa e_var_pupa f_var_pupa
     end
     
     to-report set_survival_tec
       let temp_kelvin temperature + 273.15
       let r_constant 1.987
       let a_var_egg 0.822       let b_var_egg -758.5       let c_var_egg -212100       let d_var_egg 281.9       let e_var_egg 405200       let f_var_egg 303.8
       let a_var_larva 0.758     let b_var_larva -180.2     let c_var_larva -475700     let d_var_larva 282.7     let e_var_larva 1298000    let f_var_larva 301.5
       let a_var_pupa 0.9        let b_var_pupa -73.72      let c_var_pupa -1263000     let d_var_pupa 282.5      let e_var_pupa 1095000     let f_var_pupa 306.3
       report report_survival r_constant temp_kelvin a_var_egg b_var_egg c_var_egg d_var_egg e_var_egg f_var_egg a_var_larva b_var_larva c_var_larva d_var_larva e_var_larva f_var_larva a_var_pupa b_var_pupa c_var_pupa d_var_pupa e_var_pupa f_var_pupa
     end
     
     to-report set_survival_pht
       let temp_kelvin temperature + 273.15
       let r_constant 1.987
       let a_var_egg 0.917       let b_var_egg 50       let c_var_egg -200000       let d_var_egg 283.3       let e_var_egg 400000       let f_var_egg 310.1
       let a_var_larva 0.950     let b_var_larva -150   let c_var_larva -400000     let d_var_larva 284.4     let e_var_larva 900000     let f_var_larva 310.0
       let a_var_pupa 0.960      let b_var_pupa 50      let c_var_pupa -800000      let d_var_pupa 283.1      let e_var_pupa 700000      let f_var_pupa 312.2
       report report_survival r_constant temp_kelvin a_var_egg b_var_egg c_var_egg d_var_egg e_var_egg f_var_egg a_var_larva b_var_larva c_var_larva d_var_larva e_var_larva f_var_larva a_var_pupa b_var_pupa c_var_pupa d_var_pupa e_var_pupa f_var_pupa
     end
     
     to-report set_survival_sym
       let temp_kelvin temperature + 273.15
       let r_constant 1.987
       let a_var_egg 0.834       let b_var_egg 10.94       let c_var_egg -234000       let d_var_egg 282.4       let e_var_egg 616600       let f_var_egg 304.1
       let a_var_larva 0.694     let b_var_larva -236.3    let c_var_larva -420300     let d_var_larva 283.1     let e_var_larva 1551000    let f_var_larva 305.6
       let a_var_pupa 0.882      let b_var_pupa 39.93      let c_var_pupa -992700      let d_var_pupa 282.9      let e_var_pupa 1110000     let f_var_pupa 304.7
       report report_survival r_constant temp_kelvin a_var_egg b_var_egg c_var_egg d_var_egg e_var_egg f_var_egg a_var_larva b_var_larva c_var_larva d_var_larva e_var_larva f_var_larva a_var_pupa b_var_pupa c_var_pupa d_var_pupa e_var_pupa f_var_pupa
     end
     
     to-report report_survival [r_constant temp_kelvin a_var_egg b_var_egg c_var_egg d_var_egg e_var_egg f_var_egg a_var_larva b_var_larva c_var_larva d_var_larva e_var_larva f_var_larva a_var_pupa b_var_pupa c_var_pupa d_var_pupa e_var_pupa f_var_pupa]
       let survival_egg (( a_var_egg * temp_kelvin / 298.16 ) * exp ( b_var_egg * ( 1 / 298.16 - 1 / temp_kelvin ) )) / ( 1 + exp ( ( c_var_egg / r_constant ) * ( 1 / d_var_egg - 1 / temp_kelvin ) ) + exp ( ( e_var_egg / r_constant ) * ( 1 / f_var_egg - 1 / temp_kelvin ) ))
       let survival_larva (( a_var_larva * temp_kelvin / 298.16 ) * exp ( b_var_larva * ( 1 / 298.16 - 1 / temp_kelvin ) )) / ( 1 + exp ( ( c_var_larva / r_constant ) * ( 1 / d_var_larva - 1 / temp_kelvin ) ) + exp ( ( e_var_larva / r_constant ) * ( 1 / f_var_larva - 1 / temp_kelvin ) ))
       let survival_pupa (( a_var_pupa * temp_kelvin / 298.16 ) * exp ( b_var_pupa * ( 1 / 298.16 - 1 / temp_kelvin ) )) / ( 1 + exp ( ( c_var_pupa / r_constant ) * ( 1 / d_var_pupa - 1 / temp_kelvin ) ) + exp ( ( e_var_pupa / r_constant ) * ( 1 / f_var_pupa - 1 / temp_kelvin ) ))
       let survival survival_egg * survival_larva * survival_pupa
       report survival
     end
     
     to-report demo_survival
       let mortality_dispersal 0.06
       let mortality_predation 0.145
       let days 2
       let immature_survival exp( - (mortality_dispersal + mortality_predation) * days)
       report immature_survival
     end

          ;;-----------------------------------------------------------------------------------------------------------
          ;;   INSECT SUB-MODELS : DISPERSION
          ;;-----------------------------------------------------------------------------------------------------------
     to main_insect_dispersion     ;; cf. Crespo-Perez et al. 2011 Landscape Ecology
       ;ask patches with [tec > (carrying_capacity / 1000) or sym > (carrying_capacity / 1000) or pht > (carrying_capacity / 1000)] [     ;; ### PHT and SYM
       ask patches with [tec > (carrying_capacity / 1000)] [
         
         ;; (A) fraction of insects emigrating from cells
         let vmi_tec report_vmi tec (carrying_capacity / 2) (carrying_capacity / 13)
         ;let vmi_pht report_vmi pht (carrying_capacity / 2) (carrying_capacity / 13)     ;; ### PHT and SYM
         ;let vmi_sym report_vmi sym (carrying_capacity / 2) (carrying_capacity / 13)     ;; ### PHT and SYM
         
         ;; (B) emigrating rate as a function of distance
         let dist n-values round(250 / landscape_resolution) [(? + 1) * landscape_resolution]
         let pdist (list)
         foreach dist [set pdist lput exp(0 - 0.015 * ?) pdist]
         let lengthpdisp length(pdist)   
         set pdist lput 0 pdist           
         let pdistnorm (list)            
         let i 0                         
         while [i < lengthpdisp] [
           set pdistnorm lput (item (i) pdist - item (i + 1) pdist) pdistnorm
           set i i + 1
         ]
         let pleave pdistnorm
         
         ;; (C) fraction of insects dispersing
         set disp_tec (list)
         ;set disp_pht (list)                                                                     ;; ###PHT
         ;set disp_sym (list)                                                                     ;; ###SYM
         foreach pleave [
           set disp_tec lput (vmi_tec * ? * tec) disp_tec
           ;set disp_pht lput (vmi_pht * ? * pht) disp_pht                                        ;; ###PHT
           ;set disp_sym lput (vmi_sym * ? * sym) disp_sym                                        ;; ###SYM
         ]
         
         ;; (D) applying dispersal
         let j 0
         while [j < lengthpdisp] [
           let thePatches patches with[distance myself > j and distance myself <= (j + 1)]
           ask thePatches [
             set tec tec + ([item j disp_tec] of myself / count(thePatches) / lengthpdisp)  
             ;set pht pht + ([item j disp_pht] of myself / count(thePatches) / lengthpdisp)       ;; ###PHT
             ;set sym sym + ([item j disp_sym] of myself / count(thePatches) / lengthpdisp)       ;; ###SYM
           ]
           set j j + 1
         ]
         set tec tec - (sum(disp_tec))
         ;set pht pht - (sum(disp_pht))                                                           ;; ###PHT
         ;set sym sym - (sum(disp_sym))                                                           ;; ###SYM
       ]
     end
     
     to-report report_vmi [insect var1 var2]     ;; cf. Crespo-Perez et al. 2011 Landscape Ecology
       let vmi 0.5 / (1 + exp(0 - (insect - var1) / var2))
       report vmi
     end
     
        
     
     
;---------------------------------------------------------------------------------------------------------------------
;   GUI
;---------------------------------------------------------------------------------------------------------------------     
to color_temperature
  ask patches [set pcolor scale-color red temperature -10 50]
end
to color_tec
  ask patches with [tec > (carrying_capacity / 1000)] [
  ;ask patches with [tec + sym + pht > (carrying_capacity / 1000)] [     ;; ### PHT and SYM
    if Tecia_solanivora = false [ set tec 0 ] 
    ;if Phthorimaea_operculella = false [ set pht 0 ]                                         ;; ###PHT
    ;if Symmetrischema_tangolias = false [ set sym 0 ]                                        ;; ###SYM
    set pcolor scale-color red tec 1 carrying_capacity
  ]
end
;to color_pht                                                                                 ;; ###PHT
;  ask patches with [tec + sym + pht > (carrying_capacity / 1000)] [
;    if Tecia_solanivora = false [ set tec 0 ] 
;    if Phthorimaea_operculella = false [ set pht 0 ]
;    if Symmetrischema_tangolias = false [ set sym 0 ] 
;    set pcolor scale-color green pht 1 carrying_capacity
;  ]
;end
;to color_sym                                                                                 ;; ###SYM
;  ask patches with [tec + sym + pht > (carrying_capacity / 1000)] [
;    if Tecia_solanivora = false [ set tec 0 ] 
;    if Phthorimaea_operculella = false [ set pht 0 ] 
;    if Symmetrischema_tangolias = false [ set sym 0 ] 
;    set pcolor scale-color blue sym 1 carrying_capacity
;  ]
;end
to color_storage_structure
  ask patches with [storage_structure = 1] [set pcolor blue]
end
to color_landuse
  ask patches with [landuse = 1] [set pcolor blue]
end
to color_elevation
  ask patches [set pcolor scale-color blue elevation 1000 4000]
end



;---------------------------------------------------------------------------------------------------------------------
;   OUTPUT
;---------------------------------------------------------------------------------------------------------------------   
to outputFiles [outputVariable fileName]
  if ticks = num_generations [
    let fichier ""
    ifelse repet < 10 [set fichier (word working_directory fileName "_" maj_group "_00" repet ".csv")][
      if repet < 100  [set fichier (word working_directory fileName "_" maj_group "_0"  repet ".csv")]
    ]
    file-open fichier
      ask farmers [
        file-type self file-type ";" file-type farming-style file-type ";"
        let rev reverse item 1 outputVariable
        foreach rev [file-type ? file-type ";"]
        file-print ""
      ]
    file-close
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
15
215
260
331
-1
-1
2.13
1
10
1
1
1
0
0
0
1
0
99
0
39
0
0
1
ticks
10.0

BUTTON
15
10
105
43
SETUP
setup_init
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
110
10
200
43
STEP
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
205
10
295
43
GO
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
240
420
395
453
pest view
color_tec
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
185
120
235
210
repet
1
1
0
Number

INPUTBOX
15
120
180
210
working_directory
NIL
1
0
String

SLIDER
15
85
235
118
num_generations
num_generations
history-account + 1
1000
100
1
1
NIL
HORIZONTAL

SLIDER
15
50
235
83
num_repetitions
num_repetitions
0
100
1
1
1
NIL
HORIZONTAL

SLIDER
240
50
400
83
temperature_m
temperature_m
-10
40
15
0.5
1
NIL
HORIZONTAL

CHOOSER
240
165
400
210
maj_group
maj_group
"conv" "NOrisk" "risk" "expe" "NOchange"
0

SLIDER
240
120
400
153
sd_potato_price
sd_potato_price
0
26.7
0
26.7
1
NIL
HORIZONTAL

SLIDER
240
85
400
118
sd_temperature_m
sd_temperature_m
0
10
5
5
1
NIL
HORIZONTAL

BUTTON
90
385
160
418
conv=yellow
ask turtles [\n  ask patches with [farm_owner = myself] [set pcolor 0]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"conv\"] [\n    set pcolor 45\n  ]\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
165
385
235
418
NOrisk=red
ask turtles [\n  ask patches with [farm_owner = myself] [set pcolor 0]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"NOrisk\"] [\n    set pcolor 15\n  ]\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
15
420
85
453
risk=blue
ask turtles [\n  ask patches with [farm_owner = myself] [set pcolor 0]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"risk\"] [\n    set pcolor 105\n  ]\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
90
420
160
453
expe=brown
ask turtles [\n  ask patches with [farm_owner = myself] [set pcolor 0]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"expe\"] [\n    set pcolor 35\n  ]\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
165
420
235
453
NOchange=green
ask turtles [\n  ask patches with [farm_owner = myself] [set pcolor 0]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"NOchange\"] [\n    set pcolor 55\n  ]\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
240
385
395
418
farming-type
ask turtles [\n  ask patches with [farm_owner = myself] [set pcolor 0]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"conv\"] [\n    set pcolor 45\n  ]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"NOchange\"] [\n    set pcolor 55\n  ]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"expe\"] [\n    set pcolor 35\n  ]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"NOrisk\"] [\n    set pcolor 15\n  ]\n  ask patches with [farm_owner = myself and [farming-style] of myself = \"risk\"] [\n    set pcolor 105\n  ]\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
15
385
85
418
NIL
set pcolor 9.9
NIL
1
T
PATCH
NIL
NIL
NIL
NIL
1

PLOT
240
215
400
335
network
NIL
NIL
0.0
10.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" "if ticks > 0 [\n  plot (count links with [color = 15]) / (count turtles with [farming-style = \"conv\" or farming-style = \"NOrisk\"])\n]"

MONITOR
15
335
115
380
NIL
num_farmers
17
1
11

SLIDER
120
335
235
368
world-width-def
world-width-def
0
100
100
1
1
NIL
HORIZONTAL

@#$#@#$#@
The model description follows the ODD (Overview, Design concepts, Details) protocol for describing individual- and agent-based models (Grimm et al. 2006, 2010). 

# 1. Purpose
###Question: What is the purpose of the model?
The purpose of the model is to explore the impacts of global change on the ability of a community of farmers to adapt their practices to an agricultural pest (_Tecia solanivora_ in the Northern Andes, see [Dangles _et al._ 2010]( http://dx.doi.org/10.1007/s13280-010-0041-4) for a description of the pest and the study area).

# 2. Entities, state variables, and scales	
###Questions: What kinds of entities are in the model? By what state variables, or attributes, are these entities characterized? What are the temporal and spatial resolutions and extents of the model? 

  1. **Landscape**
  1.1. _Landscape grid cells_: 
width (number of grid cells)
height (number of grid cells)
resolution for grid cells (in meters)
temperature (in °C)
land-use (short-cycle crops)
  1.2. _Pest grid cells_
pest abundance (_Tecia solanivora_)
carrying capacity for the pest
  1.3. _social system grid cells_
farm owner (name of the farmer agent)
  2. **Farmer agents**
  2.1. Farm (set of grid cells; allows calculating total farm area)
  2.2. Behavioral strategy (conventional farmer, experimenter, averse to risk farmer, risk taker or averse to change farmer)
  2.3. Current pest control practices (composed of _n_=5 different practices; allows to calculate pest control level)
  2.4. History of mean pest infestation
  2.5. History of pest control level

One time step represents approximately 6 months and simulations were run for 50 years. One grid cell represents 50 x 50 meters (0.25 ha) and the model landscape comprised 100 x 40 grid cells (_i.e._, 2,000 x 5,000 meters: 1,000 ha).

# 3. Process overview and scheduling
###Questions: Who (i.e., what entity) does what, and in what order? When are state variables updated? How is time modeled, as discrete steps or as a continuum over which both continuous processes and discrete events can occur? Except for very simple schedules, one should use pseudo-code to describe the schedule in every detail, so that the model can be re-implemented from this code. Ideally, the pseudo-code corresponds fully to the actual code used in the program implementing the ABM.

Time is represented as discrete steps of 6 months. Details of each submodel are presented in the last section of the ODD protocol.

  * The temperature is updated according to a climate change scenario (if any). 
`set_temp_preci`
  * The agricultural pest physiological submodel runs, which include reproducing according to temperature, and surviving according to temperature and predation. 
`main_insect_reproduction`
`main_insect_survival`
  * The farmers control the pest according to their pest control level, and then adapt their control practices (if possible or needed).
`farmer_adaptive_control`
  * The remaining pest disperses in the landscape (to leg eggs for the next generation), and dies.
`main_insect_dispersion`
  * Crops price on market places is updated according to economic change (if any).
`set potato_price random-normal 100 sd_potato_price`
  * Generation of output files.

# 4. Design concepts
###Questions: There are eleven design concepts. Most of these were discussed extensively by Railsback (2001) and Grimm and Railsback (2005; Chapter. 5), and are summarized here via the following questions:  
**Basic principles**. Which general concepts, theories, hypotheses, or modeling approaches are underlying the model’s design? Explain the relationship between these basic principles, the complexity expanded in this model, and the purpose of the study. How were they taken into account? Are they used at the level of submodels (_e.g._, decisions on land use, or foraging theory), or is their scope the system level (_e.g._, intermediate disturbance hypotheses)? Will the model provide insights about the basic principles themselves, _i.e._ their scope, their usefulness in real-world scenarios, validation, or modification (Grimm, 1999)? Does the model use new, or previously developed, theory for agent traits from which system dynamics emerge (_e.g._, ‘individual-based theory’ as described by Grimm and Railsback [2005; Grimm _et al._, 2005])?

The model is divided into two main submodels: an ecological submodel representing an agricultural pest evolving in a landscape (_Tecia solanivora_ in the Northern Andes), and a social submodel representing farmers with adaptive management strategies to control the pest. Both submodels are intrinsically connected to represent a social-ecological model.
The pest submodel is a cellular automaton adapted from [Crespo-Pérez _et al._, 2011](http://dx.doi.org/10.1007/s10980-011-9649-4). It is based on a physiological approach and includes reproduction, survival and dispersal of the pest over the landscape. It has been validated both in time and space with extensive field data and fully described in Crepo-Pérez _et al._, 2011.
The social submodel is an agent-based model where each agent represents a farm household, connected to others through a network. The design of the network is based on field data and previous work on the diffusion of pest control information ([Rebaudo and Dangles, 2013](http://dx.doi.org/10.1016/j.envsoft.2012.06.014); [Rebaudo and Dangles, 2011](http://dx.doi.org/10.1371/journal.pcbi.1002222)). The farmers’ adaptive management strategies are derived from field surveys and [Paredes, 2010](http://edepot.wur.nl/7207).

**Emergence**. What key results or outputs of the model are modeled as emerging from the adaptive traits, or behaviors, of individuals? In other words, what model results are expected to vary in complex and perhaps unpredictable ways when particular characteristics of individuals or their environment change? Are there other results that are more tightly imposed by model rules and hence less dependent on what individuals do, and hence ‘built in’ rather than emergent results? 

The pest control practices are emerging from the interaction between farmers and with the pest. The yield and farm income are emerging from pest control practices and the pest population dynamics.

**Adaptation**. What adaptive traits do the individuals have? What rules do they have for making decisions or changing behavior in response to changes in themselves or their environment? Do these traits explicitly seek to increase some measure of individual success regarding its objectives (e.g., “move to the cell providing fastest growth rate”, where growth is assumed to be an indicator of success; see the next concept)? Or do they instead simply cause individuals to reproduce observed behaviors (e.g., “go uphill 70% of the times”) that are implicitly assumed to indirectly convey success or fitness?  

Farmers’ adaptive traits are pest control practices. They seek to increase pest control, and subsequently yields and/or farm income, indicators of success. They change as a result of experimentation and interactions with other farmers and the pest (see details section).

**Objectives**. If adaptive traits explicitly act to increase some measure of the individual's success at meeting some objective, what exactly is that objective and how is it measured? When individuals make decisions by ranking alternatives, what criteria do they use? Some synonyms for ‘objectives’ are ‘fitness’ for organisms assumed to have adaptive traits evolved to provide reproductive success, ‘utility’ for economic reward in social models or simply ‘success criteria’. (Note that the objective of such agents as members of a team, social insects, organs—e.g., leaves—of an organism, or cells in a tissue, may not refer to themselves but to the team, colony or organism of which they are a part.) 

The objective is to increase yields and/or farm income, which are measured for each farm.

**Learning**. Many individuals or agents (but also organizations and institutions) change their adaptive traits over time as a consequence of their experience? If so, how?  

Farmers’ adaptive management take into consideration results of previous time steps in the simulation to adapt their control practices.

**Prediction**. Prediction is fundamental to successful decision-making; if an agent’s adaptive traits or learning procedures are based on estimating future consequences of decisions, how do agents predict the future conditions (either environmental or internal) they will experience? If appropriate, what internal models are agents assumed to use to estimate future conditions or consequences of their decisions? What tacit or hidden predictions are implied in these internal model assumptions? 

The tacit prediction is that changing control practices can potentially give better results in terms of yields and farm income.

**Sensing**. What internal and environmental state variables are individuals assumed to sense and consider in their decisions? What state variables of which other individuals and entities can an individual perceive; for example, signals that another individual may intentionally or unintentionally send? Sensing is often assumed to be local, but can happen through networks or can even be assumed to be global (e.g., a forager on one site sensing the resource levels of all other sites it could move to). If agents sense each other through social networks, is the structure of the network imposed or emergent? Are the mechanisms by which agents obtain information modeled explicitly, or are individuals simply assumed to know these variables? 

Farmers sense yields and/or farm income, and those of other farmers located in their social network. The social network is composed of neighboring farms. They also perceive control practices of their neighbors, but not effective pest control. The structure of the social network is imposed and limited to neighbors, but the localization of farmers is initialized randomly according to a Vornoï tessellation of the landscape.

**Interaction**. What kinds of interactions among agents are assumed? Are there direct interactions in which individuals encounter and affect others, or are interactions indirect, e.g., via competition for a mediating resource? If the interactions involve communication, how are such communications represented? 

Interactions between farmers are direct and affect control practices for farmers whom strategy is based on mimicking. 

**Stochasticity**. What processes are modeled by assuming they are random or partly random? Is stochasticity used, for example, to reproduce variability in processes for which it is unimportant to model the actual causes of the variability? Is it used to cause model events or behaviors to occur with a specified frequency? 

Localization of farmers and farms area are initialized randomly according to a Vornoï tessellation. Global change scenarios introduce stochasticity in temperature, potato price and proportion of the different adaptive management strategies (_i.e._, to reproduce variability in processes for which it is unimportant to model the actual causes of the variability).

**Collectives**. Do the individuals form or belong to aggregations that affect, and are affected by, the individuals? Such collectives can be an important intermediate level of organization in an ABM; examples include social groups, fish schools and bird flocks, and human networks and organizations. How are collectives represented? Is a particular collective an emergent property of the individuals, such as a flock of birds that assembles as a result of individual behaviors, or is the collective simply a definition by the modeler, such as the set of individuals with certain properties, defined as a separate kind of entity with its own state variables and traits?

All farmers in the landscape belong to an aggregate called community. 

**Observation**. What data are collected from the ABM for testing, understanding, and analyzing it, and how and when are they collected? Are all output data freely used, or are only certain data sampled and used, to imitate what can be observed in an empirical study (“Virtual Ecologist” approach; Zurell et al., 2010)? 

Collected data from the ABM are control practices, pest control level, yields and farm income for each farmer at each model generation.

# 5. Initialization
###Questions: What is the initial state of the model world, _i.e._, at time t = 0 of a simulation run? In detail, how many entities of what type are there initially, and what are the exact values of their state variables (or how were they set stochastically)? Is initialization always the same, or is it allowed to vary among simulations? Are the initial values chosen arbitrarily or based on data? 

The initial state of the model is a landscape of 100 x 40 grid cells with 600 farmers. The number of generations is set to 100 (corresponding to 50 years), the temperature to 15°C, without variation in temperature and in potato price. The dominant adaptive management strategy is that of conventional farmers. All these variables were studied by performing sensitivity analyses to analyze consequences of their initial state. The farmers control practices are initialized to represent the distribution observed in Northern Andean communities of farmers (see Fig. 2A in [Rebaudo and Dangles, 2011](http://dx.doi.org/10.1371/journal.pcbi.1002222)). They are located randomly in the landscape and farms built according to a Vornoï tessellation. 

# 6. Input data
Question: Does the model use input from external sources such as data files or other models to represent processes that change over time?

The model does not use input from external sources. Global change is represented within the model.

# 7. Submodels
###Questions: What, in detail, are the submodels that represent the processes listed in ‘Process overview and scheduling’? What are the model parameters, their dimensions, and reference values? How were submodels designed or chosen, and how were they parameterized and then tested?

  * `set_temp_preci`

This submodel updates the temperature according to the climate change scenario (if any). The new temperature is chosen randomly in a Gaussian distribution of parameters _mean_=15 and _sd_=5, in order to allow temperatures outside of the range of the pest survival.

  * `main_insect_reproduction`

For a complete description please refer to [Crespo-Pérez _et al._, 2011](http://dx.doi.org/10.1007/s10980-011-9649-4). Briefly, the number of offspring is temperature dependant:

    let fecundity o + (p * exp (- (temperature - q) / r) * ((((temperature - q) / r) + (s - 1)) / (s - 1))^(s - 1)) 
    with o=-30.118, p=343.281, q=18.753, r=17.109,  and  s=3.300.

  * `main_insect_survival`

First the submodel computes the temperature dependant mortality based on the Sharpe and de Michel equation (see [Crespo-Pérez _et al._, 2011](http://dx.doi.org/10.1007/s10980-011-9649-4)) using `set_survival_tec`. 
Then the submodel computes the demographically based mortality, using `demo_survival`.
Finally, the submodel computes a possible adjustment (when various species are present, so that one time step always correspond to 6 months) and a possible precipitations dependant mortality (not used here). Pest populations are then updated according to mortality.

  * `farmer_adaptive_control`

First, farmers control the pest according to their control practices. The control practices are compared to a theoretical optimum and the absolute difference between practices and optimal practices determines pest control information (in percent of maximum pest control information). 

    IF control practices = [15 62 12 89 88] 
      and theoretical optimum = [50 50 50 50 50]
    THEN pest control level = mean[35 12 38 39 38] = 32.4

Then this pest control information is ranged from 0 to 5 to fit the correlation between information and effective control based on [Rebaudo and Dangles, 2011](http://dx.doi.org/10.1371/journal.pcbi.1002222), and the number of pest is updated accordingly:

    set tec tec - tec * (0.15 * mycontrol + 0.21 + epsilon) 
    ;; here epsilon is null

Then, the history of pest infestation and pest control level are stored for each farmer. This first step is a fixed behavior (opposed to adaptive behavior), assuming that farmers always have to control the pest the best they can and at every crop cycle.

Second, farmers assessed their situation according to their adaptive behavior strategy:

**Averse to risk farmers** look for another farmer in their network which has better yields over the last 2 years in comparison to themselves and adapt their control practices randomly toward this other farmer (if any). The intensity of change in control practices is of 2 points over 5 control practices ranging from 0 to 100 and a theoretical optimum of 50, derived from a theoretical basis and from [Rebaudo and Dangles, 2011](http://dx.doi.org/10.1371/journal.pcbi.1002222).

    IF any(farmer in social network) with(last 2 years yields) > my(last 2 years yields)
    THEN changing my practices according to this farmer

**Conventional farmers** look for another farmer in their network which has better farm income over the last 2 years in comparison to themselves and adapt their control practices randomly toward this other farmer (if any). The intensity of change in control practices is of 4 points over 5 control practices ranging from 0 to 100 and a theoretical optimum of 50, derived from a theoretical basis and from [Rebaudo and Dangles, 2011](http://dx.doi.org/10.1371/journal.pcbi.1002222).

    IF any(farmer in social network) with(last 2 years farm income) > my(last 2 years farm income)
    THEN changing my practices according to this farmer

**Risk takers farmers**, relying on external recommendations, compare their own last farm income with their current farm income. If the last farm income is higher, then the recommendation was not adapted to their situation, so that they return to the previous practices and try new recommendations. Else, the recommendation was adapted to their situation, so that they keep going with the recommendation and continue with new ones. The intensity of change in control practices is of 10 points over 5 control practices ranging from 0 to 100 and a theoretical optimum of 50, derived from a theoretical basis and from [Rebaudo and Dangles, 2011](http://dx.doi.org/10.1371/journal.pcbi.1002222).

**Experimenters** compare their own last yields with their current yields. If the last yields are higher than actual yields, then the experimentation was not adapted to their situation, so that they return to the previous control practices and try new experimentations. Else, the experimentation gives good results, so that they keep new practices and try another experiment. The intensity of change in control practices is of 8 points over 5 control practices ranging from 0 to 100 and a theoretical optimum of 50, derived from a theoretical basis and from [Rebaudo and Dangles, 2011](http://dx.doi.org/10.1371/journal.pcbi.1002222).

**Averse to change farmers** do not change their control practices.

  * `main_insect_dispersion`
Pest dispersion is based on [Crespo-Pérez _et al._, 2011](http://dx.doi.org/10.1007/s10980-011-9649-4). Briefly, it is density dependant and pest can disperse up to 250 meters.


# CREDITS AND REFERENCES

This program is licensed under the Creative Commons (Copyright (c) IRD / François Rebaudo) except the code adapted from David O'Sullivan and George Perry licensed under the Creative Commons (Copyright (c) 2011-13 David O'Sullivan and George Perry : O'Sullivan D and Perry GLW 2013 Spatial Simulation: Exploring Pattern and Process. Wiley, Chichester, England).
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
